/*
 * This software is in the public domain.
 * You may use it as you see fit with the understanding that
 * we make no guarantees of its fitness for any particular purpose.
 *
 * Author: XESS Corporation
 *         www.xess.com
 */

//
// The XC95 object specializes the XCPORT object so that the XC9500 CPLD on the
// XS95 Board can be configured and the RAM can be loaded.  The XC95 object controls
// the JTAG port of the XC9500 chip.
//

// $Log: xc95.h $
// Revision 1.2  1999/03/30 18:27:41  DEVB
// Released source into the public domain
//
// Revision 1.1  1999/03/29 22:59:47  DEVB
// Initial revision
//

#ifndef XC95_H
#define XC95_H

#include "xcport.h"

// definitions of XC9500 CPLD types
enum{
	XC9572Type = 0,
	XC95108Type = 1,
	UnknownXC9500Type = 2,
};

class XC95 : public XCPort
{
	public:

	/* begin XC95 methods */

	/* method: XC95 */
	// instantiate an object for loading an XS95 board on the parallel port
	 XC95(XSError* err, unsigned int portNum);

	/* method: ConfigureForRAMDownload */
	// configure the CPLD so it can interface to the XS Board RAM
	bool ConfigureForRAMDownload();

	/* method: ProgramEEPROM */
	// program an Atmel serial EEPROM with a bitstream
	bool ProgramEEPROM(istream& is);

	/* method: ConfigureFPLD */
	// Send out a byte of configuration information to the XC9500 Flash
	void ConfigureFPLD(unsigned char b);

	private:

	/* method: InitConfigureFPLD */
	// Initialize the XC9500 CPLD for configuration through the JTAG interface
	void InitConfigureFPLD(void);

	/* method: ConfigureFPLD */
	// Process SVF and send results through JTAG configuration pins
	bool ConfigureFPLD(istream& is);

	/* method: DetectXC9500Type */
	// detect the presence and type of the XC9500 CPLD chip on the port
	bool DetectXC9500Type();

	/* end XC95 methods */

	private:
};

/* begin XC95 functions */

/* function: NextWord */
static string NextWord(string& s);

/* end XC95 functions */

#endif
