/*
 * This software is in the public domain.
 * You may use it as you see fit with the understanding that
 * we make no guarantees of its fitness for any particular purpose.
 *
 * Author: XESS Corporation
 *         www.xess.com
 */

//
// The XSPORT program lets the user force binary values onto the
// data pins of the parallel port
//

// $Log: xsport.cpp $
// Revision 1.3  1999/03/30 19:46:28  DEVB
// Added code to display release version number
//
// Revision 1.2  1999/03/30 18:27:12  DEVB
// Released source into the public domain
//
// Revision 1.1  1999/03/29 22:07:04  DEVB
// Initial revision
//

#include <stdlib.h>
#include <assert.h>
#ifdef __WIN32__
#include <strstrea.h>
#else
#include <strstream.h>
#endif
#include <fstream.h>
#include <string>
using std::string;

/*
#include <iomanip.h>
#include <time.h>
#include <direct.h>
*/

#include "pport.h"
#include "utils.h"

//
// global variables
//
string version("$Name: V2_0_0 $");
XSError errMsg(cerr); // setup error channel
unsigned int portNum = 1; // default parallel port number
int portAddress = -1; // port address starts out unset

//
// print a string that shows the options and how to use this program.
//
void Usage(const string& progName)
{
	cerr << progName.data() << " [-[h|help]] [-[p|port] #[1,2,3]] <binary number>" << endl;
	int pos = version.find("Name:");
	if(pos < 0)
	{
		cerr << "Version number unavailable!!" << endl;
	}
	else
	{
		string versionNumber = version.substr(pos+5,version.length()-pos-5);
		for(int i=versionNumber.length()-1; i>=0; i--)
		{
			if(versionNumber[i]=='_')
				versionNumber[i] = '.';
			else if(versionNumber[i]=='$')
				versionNumber[i] = ' ';
		}
		cerr << "Version " << versionNumber.data() << endl;
	}
}

//
// process the command line options and set the appropriate flags.
//
void ProcessOpts(int* argc, char** argv)
{
	for(int i=1; i< *argc; i++)
	{
		string arg = argv[i];
		ConvertToUpperCase(arg);
		
		if(arg[0] == '-')  // found an option
		{
			int cnt, j;
			
			// ask for help but we don't give much
			if(arg=="-H" || arg=="-HELP")
			{
				Usage( string(argv[0]) );
				cnt = 1;
			}
			
			// specify the printer port being used
			else if(arg=="-PORT" || arg=="-P")
			{
				istrstream argstr(argv[i+1]);
				argstr >> portNum;
				cnt = 2;
			}
			
			// specify the I/O address of the parallel port
			// (not currently used)
			else if(arg=="-A" || arg=="-ADDRESS")
			{
				istrstream argstr(argv[i+1]);
				argstr >> portAddress;
				cnt = 2;
			}
			
			// can't figure out what the user wants
			else
			{
				errMsg.SetSeverity(XSErrorMinor);
				errMsg << "unknown option: " << argv[i] << endl;
				errMsg.EndMsg();
				Usage(string(argv[0]));
				cnt = 1;
			}
			
			// remove the processed options from the argument list
			for(j=i+cnt; j< *argc; j++)
				argv[j-cnt] = argv[j];
			*argc -= cnt;
			i--;
		}
	}
}


main( int argc, char **argv )
{
	// set the error message header to the name of this program
	string cmd = argv[0];
	cmd = StripSuffix(StripPrefix(cmd));
	errMsg.SetHeader(cmd);
	
	ProcessOpts( &argc, argv );  // process command line options
	if(errMsg.IsError()) // exit if errors in options
		exit(1);
	if(argc<2)	// there has to be a binary string of bits
		exit(1);
	
	int portValue = 0;			// initialize value to place on parallel port
	string bits = argv[1];		// get the string of binary bits
	int i = bits.length()-1;
	for( unsigned int bitMask=1; (i>=0) && (bitMask!=256); bitMask<<=1,i-- )
	{	// iterate through the string while assigning binary values to
		// the parallel port data bits
		switch( bits[i] )
		{
			// binary one is represented by '1', 'h', or 'H'
		case '1':
		case 'H':
		case 'h':
			portValue |= bitMask;
			break;
			// binary zero is represented by '0', 'l', or 'L'
		case '0':
		case 'L':
		case 'l':
			portValue &= ~bitMask;
			break;
			// erroneous binary value if we get here!!
		default:
			errMsg.SetSeverity(XSErrorMinor);
			errMsg << "illegal binary value in " << bits.data() << endl;
			errMsg.EndMsg();
			break;
		}
	}

	// exit without changing the parallel port if there was an error
	if(errMsg.IsError())
		exit(3);
	// otherwise, create the parallel port object
	// (remember to inver the lower two data bits due to the XS Board inverters)
	Pport port(&errMsg,portNum,0x03);
	// now place the new port value on the parallel port pins
	port.SetData(portValue);	
	}
